@@ -2,7 +2,7 @@
 #of this repository contains the full copyright notices and license terms.
 "Purchase"
 from trytond.model import ModelWorkflow, ModelView, ModelSQL, fields
-from trytond.report import CompanyReport
+from trytond.modules.company import CompanyReport
 from trytond.wizard import Wizard
 from trytond.backend import TableHandler
 from decimal import Decimal
@@ -22,7 +22,8 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     company = fields.Many2One('company.company', 'Company', required=True,
             states={
                 'readonly': "state != 'draft' or bool(lines)",
-            }, domain="[('id', '=', context.get('company', 0))]")
+            }, domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
     reference = fields.Char('Reference', size=None, readonly=True, select=1)
     supplier_reference = fields.Char('Supplier Reference', select=1)
     description = fields.Char('Description', size=None, states=_STATES)
@@ -42,7 +43,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     party_lang = fields.Function('get_function_fields', type='char',
             string='Party Language', on_change_with=['party'])
     invoice_address = fields.Many2One('party.address', 'Invoice Address',
-            domain="[('party', '=', party)]", states=_STATES)
+            domain=["('party', '=', party)"], states=_STATES)
     warehouse = fields.Many2One('stock.location', 'Warehouse',
             domain=[('type', '=', 'warehouse')], required=True, states=_STATES)
     currency = fields.Many2One('currency.currency', 'Currency', required=True,
@@ -63,7 +64,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     invoice_method = fields.Selection([
         ('manual', 'Manual'),
         ('order', 'Based On Order'),
-        ('packing', 'Based On Shipment'),
+        ('shipment', 'Based On Shipment'),
     ], 'Invoice Method', required=True, states=_STATES)
     invoice_state = fields.Selection([
         ('none', 'None'),
@@ -83,19 +84,19 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             string='Invoices Paid')
     invoice_exception = fields.Function('get_function_fields', type='boolean',
             string='Invoices Exception')
-    packing_state = fields.Selection([
+    shipment_state = fields.Selection([
         ('none', 'None'),
         ('waiting', 'Waiting'),
         ('received', 'Received'),
         ('exception', 'Exception'),
     ], 'Shipment State', readonly=True, required=True)
-    packings = fields.Function('get_function_fields', type='many2many',
-            relation='stock.packing.in', string='Shipments')
+    shipments = fields.Function('get_function_fields', type='many2many',
+            relation='stock.shipment.in', string='Shipments')
     moves = fields.Function('get_function_fields', type='many2many',
             relation='stock.move', string='Moves')
-    packing_done = fields.Function('get_function_fields', type='boolean',
+    shipment_done = fields.Function('get_function_fields', type='boolean',
             string='Shipment Done')
-    packing_exception = fields.Function('get_function_fields', type='boolean',
+    shipment_exception = fields.Function('get_function_fields', type='boolean',
             string='Shipments Exception')
 
     def __init__(self):
@@ -109,6 +110,29 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                         'an "Account Payable" on the party "%s"!',
             })
 
+    def init(self, cursor, module_name):
+        # Migration from 1.2: packing renamed into shipment
+        cursor.execute("UPDATE ir_model_data "\
+                "SET fs_id = REPLACE(fs_id, 'packing', 'shipment') "\
+                "WHERE fs_id like '%%packing%%' AND module = %s",
+                (module_name,))
+        cursor.execute("UPDATE ir_model_field "\
+                "SET relation = REPLACE(relation, 'packing', 'shipment'), "\
+                    "name = REPLACE(name, 'packing', 'shipment') "
+                "WHERE (relation like '%%packing%%' "\
+                    "OR name like '%%packing%%') AND module = %s",
+                (module_name,))
+        table = TableHandler(cursor, self, module_name)
+        table.column_rename('packing_state', 'shipment_state')
+
+        super(Purchase, self).init(cursor, module_name)
+
+        # Migration from 1.2: rename packing to shipment in
+        # invoice_method values
+        cursor.execute("UPDATE " + self._table + " "\
+                "SET invoice_method = 'shipment' "\
+                "WHERE invoice_method = 'packing'")
+
     def default_payment_term(self, cursor, user, context=None):
         payment_term_obj = self.pool.get('account.invoice.payment_term')
         payment_term_ids = payment_term_obj.search(cursor, user,
@@ -169,7 +193,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     def default_invoice_state(self, cursor, user, context=None):
         return 'none'
 
-    def default_packing_state(self, cursor, user, context=None):
+    def default_shipment_state(self, cursor, user, context=None):
         return 'none'
 
     def on_change_party(self, cursor, user, ids, vals, context=None):
@@ -251,6 +275,8 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
     def on_change_lines(self, cursor, user, ids, vals, context=None):
         currency_obj = self.pool.get('currency.currency')
         tax_obj = self.pool.get('account.tax')
+        invoice_obj = self.pool.get('account.invoice')
+
         if context is None:
             context = {}
         res = {
@@ -266,6 +292,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             ctx = context.copy()
             ctx.update(self.get_tax_context(cursor, user, vals,
                 context=context))
+            taxes = {}
             for line in vals['lines']:
                 if line.get('type', 'line') != 'line':
                     continue
@@ -274,7 +301,16 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 for tax in tax_obj.compute(cursor, user, line.get('taxes', []),
                         line.get('unit_price', Decimal('0.0')),
                         line.get('quantity', 0.0), context=context):
-                    res['tax_amount'] += tax['amount']
+                    key, val = invoice_obj._compute_tax(cursor, user, tax,
+                            'in_invoice', context=context)
+                    if not key in taxes:
+                        taxes[key] = val['amount']
+                    else:
+                        taxes[key] += val['amount']
+            if currency:
+                for key in taxes:
+                    res['tax_amount'] += currency_obj.round(cursor, user,
+                            currency, taxes[key])
         if currency:
             res['untaxed_amount'] = currency_obj.round(cursor, user, currency,
                     res['untaxed_amount'])
@@ -322,17 +358,17 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         if 'invoice_exception' in names:
             res['invoice_exception'] = self.get_invoice_exception(cursor, user,
                     purchases, context=context)
-        if 'packings' in names:
-            res['packings'] = self.get_packings(cursor, user, purchases,
+        if 'shipments' in names:
+            res['shipments'] = self.get_shipments(cursor, user, purchases,
                     context=context)
         if 'moves' in names:
             res['moves'] = self.get_moves(cursor, user, purchases,
                     context=context)
-        if 'packing_done' in names:
-            res['packing_done'] = self.get_packing_done(cursor, user,
+        if 'shipment_done' in names:
+            res['shipment_done'] = self.get_shipment_done(cursor, user,
                     purchases, context=context)
-        if 'packing_exception' in names:
-            res['packing_exception'] = self.get_packing_exception(cursor,
+        if 'shipment_exception' in names:
+            res['shipment_exception'] = self.get_shipment_exception(cursor,
                     user, purchases, context=context)
         return res
 
@@ -391,6 +427,8 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         '''
         currency_obj = self.pool.get('currency.currency')
         tax_obj = self.pool.get('account.tax')
+        invoice_obj = self.pool.get('account.invoice')
+
         if context is None:
             context = {}
         res = {}
@@ -399,6 +437,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             ctx.update(self.get_tax_context(cursor, user,
                 purchase, context=context))
             res.setdefault(purchase.id, Decimal('0.0'))
+            taxes = {}
             for line in purchase.lines:
                 if line.type != 'line':
                     continue
@@ -406,7 +445,15 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 for tax in tax_obj.compute(
                     cursor, user, [t.id for t in line.taxes], line.unit_price,
                     line.quantity, context=ctx):
-                    res[purchase.id] += tax['amount']
+                    key, val = invoice_obj._compute_tax(cursor, user, tax,
+                            'in_invoice', context=context)
+                    if not key in taxes:
+                        taxes[key] = val['amount']
+                    else:
+                        taxes[key] += val['amount']
+            for key in taxes:
+                res[purchase.id] += currency_obj.round(cursor, user,
+                        purchase.currency, taxes[key])
             res[purchase.id] = currency_obj.round(cursor, user, purchase.currency,
                     res[purchase.id])
         return res
@@ -481,25 +528,25 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             res[purchase.id] = val
         return res
 
-    def get_packings(self, cursor, user, purchases, context=None):
+    def get_shipments(self, cursor, user, purchases, context=None):
         '''
-        Return the packings for the purchases.
+        Return the shipments for the purchases.
 
         :param cursor: the database cursor
         :param user: the user id
         :param purchases: a BrowseRecordList of purchases
         :param context: the context
         :return: a dictionary with purchase id as key and
-            a list of packing_in id as value
+            a list of shipment_in id as value
         '''
         res = {}
         for purchase in purchases:
             res[purchase.id] = []
             for line in purchase.lines:
                 for move in line.moves:
-                    if move.packing_in:
-                        if move.packing_in.id not in res[purchase.id]:
-                            res[purchase.id].append(move.packing_in.id)
+                    if move.shipment_in:
+                        if move.shipment_in.id not in res[purchase.id]:
+                            res[purchase.id].append(move.shipment_in.id)
         return res
 
     def get_moves(self, cursor, user, purchases, context=None):
@@ -520,7 +567,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 res[purchase.id].extend([x.id for x in line.moves])
         return res
 
-    def get_packing_done(self, cursor, user, purchases, context=None):
+    def get_shipment_done(self, cursor, user, purchases, context=None):
         '''
         Return if all the move have been done for the purchases
 
@@ -541,9 +588,9 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
             res[purchase.id] = val
         return res
 
-    def get_packing_exception(self, cursor, user, purchases, context=None):
+    def get_shipment_exception(self, cursor, user, purchases, context=None):
         '''
-        Return if there is a packing in exception for the purchases
+        Return if there is a shipment in exception for the purchases
 
         :param cursor: the database cursor
         :param user: the user id
@@ -595,7 +642,7 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         default['invoice_state'] = 'none'
         default['invoices'] = False
         default['invoices_ignored'] = False
-        default['packing_state'] = 'none'
+        default['shipment_state'] = 'none'
         return super(Purchase, self).copy(cursor, user, ids, default=default,
                 context=context)
 
@@ -649,6 +696,39 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
                 res[line.id] = val
         return res
 
+    def _get_invoice_purchase(self, cursor, user, purchase, context=None):
+        '''
+        Return invoice values for purchase
+
+        :param cursor: the database cursor
+        :param user: the user id
+        :param purchase: the BrowseRecord of the purchase
+        :param context: the context
+
+        :return: a dictionary with purchase fields as key and
+            purchase values as value
+        '''
+        journal_obj = self.pool.get('account.journal')
+
+        journal_id = journal_obj.search(cursor, user, [
+            ('type', '=', 'expense'),
+            ], limit=1, context=context)
+        if journal_id:
+            journal_id = journal_id[0]
+
+        res = {
+            'company': purchase.company.id,
+            'type': 'in_invoice',
+            'reference': purchase.reference,
+            'journal': journal_id,
+            'party': purchase.party.id,
+            'invoice_address': purchase.invoice_address.id,
+            'currency': purchase.currency.id,
+            'account': purchase.party.account_payable.id,
+            'payment_term': purchase.payment_term.id,
+        }
+        return res
+
     def create_invoice(self, cursor, user, purchase_id, context=None):
         '''
         Create invoice for the purchase id
@@ -660,7 +740,6 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         :return: the id of the invoice or None
         '''
         invoice_obj = self.pool.get('account.invoice')
-        journal_obj = self.pool.get('account.journal')
         invoice_line_obj = self.pool.get('account.invoice.line')
         purchase_line_obj = self.pool.get('purchase.line')
 
@@ -678,25 +757,10 @@ class Purchase(ModelWorkflow, ModelSQL, ModelView):
         if not invoice_lines:
             return
 
-        journal_id = journal_obj.search(cursor, user, [
-            ('type', '=', 'expense'),
-            ], limit=1, context=context)
-        if journal_id:
-            journal_id = journal_id[0]
-
         ctx = context.copy()
         ctx['user'] = user
-        invoice_id = invoice_obj.create(cursor, 0, {
-            'company': purchase.company.id,
-            'type': 'in_invoice',
-            'reference': purchase.reference,
-            'journal': journal_id,
-            'party': purchase.party.id,
-            'invoice_address': purchase.invoice_address.id,
-            'currency': purchase.currency.id,
-            'account': purchase.party.account_payable.id,
-            'payment_term': purchase.payment_term.id,
-        }, context=ctx)
+        vals = self._get_invoice_purchase(cursor, user, purchase, context=context)
+        invoice_id = invoice_obj.create(cursor, 0, vals, context=ctx)
 
         for line_id in invoice_lines:
             for vals in invoice_lines[line_id]:
@@ -794,8 +858,8 @@ class PurchaseLine(ModelSQL, ModelView):
                 'required': "product",
                 'invisible': "type != 'line'",
                 'readonly': "not globals().get('_parent_purchase')",
-            }, domain="[('category', '=', " \
-                    "(product, 'product.default_uom.category'))]",
+            }, domain=["('category', '=', " \
+                    "(product, 'product.default_uom.category'))"],
             context="{'category': (product, 'product.default_uom.category')}",
             on_change=['product', 'quantity', 'unit', '_parent_purchase.currency',
                 '_parent_purchase.party'])
@@ -1296,10 +1360,9 @@ class Template(ModelSQL, ModelView):
         'readonly': "active == False",
         'invisible': "not purchasable",
         'required': "purchasable",
-        }, domain="[('category', '=', " \
-                "(default_uom, 'uom.category'))]",
+        }, domain=["('category', '=', (default_uom, 'uom.category'))"],
         context="{'category': (default_uom, 'uom.category')}",
-        on_change_with=['default_uom', 'purchase_uom'])
+        on_change_with=['default_uom', 'purchase_uom', 'purchasable'])
 
     def __init__(self):
         super(Template, self).__init__()
@@ -1307,7 +1370,28 @@ class Template(ModelSQL, ModelView):
                 'change_purchase_uom': 'Purchase prices are based on the purchase uom, '\
                     'are you sure to change it?',
             })
-
+        if 'not bool(account_category) and bool(purchasable)' not in \
+                self.account_expense.states.get('required', ''):
+            self.account_expense = copy.copy(self.account_expense)
+            self.account_expense.states = copy.copy(self.account_expense.states)
+            if not self.account_expense.states.get('required'):
+                self.account_expense.states['required'] = \
+                        "not bool(account_category) and bool(purchasable)"
+            else:
+                self.account_expense.states['required'] = '(' + \
+                        self.account_expense.states['required'] + ') ' \
+                        'or (not bool(account_category) and bool(purchasable))'
+        if 'account_category' not in self.account_expense.depends:
+            self.account_expense = copy.copy(self.account_expense)
+            self.account_expense.depends = \
+                    copy.copy(self.account_expense.depends)
+            self.account_expense.depends.append('account_category')
+        if 'purchasable' not in self.account_expense.depends:
+            self.account_expense = copy.copy(self.account_expense)
+            self.account_expense.depends = \
+                    copy.copy(self.account_expense.depends)
+            self.account_expense.depends.append('purchasable')
+        self._reset_columns()
 
     def default_purchasable(self, cursor, user, context=None):
         if context is None:
@@ -1442,7 +1526,8 @@ class ProductSupplier(ModelSQL, ModelView):
             'product_supplier', 'Prices')
     company = fields.Many2One('company.company', 'Company', required=True,
             ondelete='CASCADE', select=1,
-            domain="[('id', '=', context.get('company', 0))]")
+            domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
     delivery_time = fields.Integer('Delivery Time',
             help="In number of days")
 
@@ -1529,11 +1614,11 @@ class ProductSupplierPrice(ModelSQL, ModelView):
 ProductSupplierPrice()
 
 
-class PackingIn(ModelSQL, ModelView):
-    _name = 'stock.packing.in'
+class ShipmentIn(ModelSQL, ModelView):
+    _name = 'stock.shipment.in'
 
     def __init__(self):
-        super(PackingIn, self).__init__()
+        super(ShipmentIn, self).__init__()
         self.incoming_moves = copy.copy(self.incoming_moves)
         if "('supplier', '=', supplier)" not in self.incoming_moves.add_remove:
             self.incoming_moves.add_remove = "[" + \
@@ -1550,7 +1635,7 @@ class PackingIn(ModelSQL, ModelView):
         purchase_obj = self.pool.get('purchase.purchase')
         purchase_line_obj = self.pool.get('purchase.line')
 
-        res = super(PackingIn, self).write(cursor, user, ids, vals,
+        res = super(ShipmentIn, self).write(cursor, user, ids, vals,
                 context=context)
 
         if 'state' in vals and vals['state'] in ('received', 'cancel'):
@@ -1558,8 +1643,8 @@ class PackingIn(ModelSQL, ModelView):
             move_ids = []
             if isinstance(ids, (int, long)):
                 ids = [ids]
-            for packing in self.browse(cursor, user, ids, context=context):
-                move_ids.extend([x.id for x in packing.incoming_moves])
+            for shipment in self.browse(cursor, user, ids, context=context):
+                move_ids.extend([x.id for x in shipment.incoming_moves])
 
             purchase_line_ids = purchase_line_obj.search(cursor, user, [
                 ('moves', 'in', move_ids),
@@ -1571,19 +1656,19 @@ class PackingIn(ModelSQL, ModelView):
                         purchase_ids.append(purchase_line.purchase.id)
 
             purchase_obj.workflow_trigger_validate(cursor, user, purchase_ids,
-                    'packing_update', context=context)
+                    'shipment_update', context=context)
         return res
 
     def button_draft(self, cursor, user, ids, context=None):
-        for packing in self.browse(cursor, user, ids, context=context):
-            for move in packing.incoming_moves:
+        for shipment in self.browse(cursor, user, ids, context=context):
+            for move in shipment.incoming_moves:
                 if move.state == 'cancel' and move.purchase_line:
                     self.raise_user_error(cursor, 'reset_move')
 
-        return super(PackingIn, self).button_draft(
+        return super(ShipmentIn, self).button_draft(
             cursor, user, ids, context=context)
 
-PackingIn()
+ShipmentIn()
 
 
 class Move(ModelSQL, ModelView):
@@ -1596,33 +1681,36 @@ class Move(ModelSQL, ModelView):
     purchase = fields.Function('get_purchase', type='many2one',
             relation='purchase.purchase', string='Purchase',
             fnct_search='search_purchase', select=1, states={
-                'invisible': "type != 'input'",
-            })
+                'invisible': "not purchase_visible",
+            }, depends=['purchase_visible'])
     purchase_quantity = fields.Function('get_purchase_fields',
             type='float', digits="(16, unit_digits)",
             string='Purchase Quantity',
             states={
-                'invisible': "type != 'input'",
-            })
+                'invisible': "not purchase_visible",
+            }, depends=['purchase_visible'])
     purchase_unit = fields.Function('get_purchase_fields',
             type='many2one', relation='product.uom',
             string='Purchase Unit',
             states={
-                'invisible': "type != 'input'",
-            })
+                'invisible': "not purchase_visible",
+            }, depends=['purchase_visible'])
     purchase_unit_digits = fields.Function('get_purchase_fields',
             type='integer', string='Purchase Unit Digits')
     purchase_unit_price = fields.Function('get_purchase_fields',
             type='numeric', digits=(16, 4), string='Purchase Unit Price',
             states={
-                'invisible': "type != 'input'",
-            })
+                'invisible': "not purchase_visible",
+            }, depends=['purchase_visible'])
     purchase_currency = fields.Function('get_purchase_fields',
             type='many2one', relation='currency.currency',
             string='Purchase Currency',
             states={
-                'invisible': "type != 'input'",
-            })
+                'invisible': "not purchase_visible",
+            }, depends=['purchase_visible'])
+    purchase_visible = fields.Function('get_purchase_visible',
+            type="boolean", string='Purchase Visible',
+            on_change_with=['from_location'])
     supplier = fields.Function('get_supplier', type='many2one',
             relation='party.party', string='Supplier',
             fnct_search='search_supplier', select=1)
@@ -1687,6 +1775,33 @@ class Move(ModelSQL, ModelView):
                         res[name][move.id] = move.purchase_line[name[9:]].id
         return res
 
+    def default_purchase_visible(self, cursor, user, context=None):
+        from_location = self.default_from_location(cursor, user,
+                context=context)
+        vals = {
+            'from_location': from_location,
+        }
+        return self.on_change_with_purchase_visible(cursor, user, [], vals,
+                context=context)
+
+    def on_change_with_purchase_visible(self, cursor, user, ids, vals,
+            context=None):
+        location_obj = self.pool.get('stock.location')
+        if vals.get('from_location'):
+            from_location = location_obj.browse(cursor, user,
+                    vals['from_location'], context=context)
+            if from_location.type == 'supplier':
+                return True
+        return False
+
+    def get_purchase_visible(self, cursor, user, ids, name, arg, context=None):
+        res = {}
+        for move in self.browse(cursor, user, ids, context=context):
+            res[move.id] = False
+            if move.from_location.type == 'supplier':
+                res[move.id] = True
+        return res
+
     def get_supplier(self, cursor, user, ids, name, arg, context=None):
         res = {}
         for move in self.browse(cursor, user, ids, context=context):
@@ -1723,7 +1838,7 @@ class Move(ModelSQL, ModelView):
                     purchase_ids.add(purchase_line.purchase.id)
             if purchase_ids:
                 purchase_obj.workflow_trigger_validate(cursor, user,
-                        list(purchase_ids), 'packing_update', context=context)
+                        list(purchase_ids), 'shipment_update', context=context)
         return res
 
     def delete(self, cursor, user, ids, context=None):
@@ -1746,7 +1861,7 @@ class Move(ModelSQL, ModelView):
                 purchase_ids.add(purchase_line.purchase.id)
             if purchase_ids:
                 purchase_obj.workflow_trigger_validate(cursor, user,
-                        list(purchase_ids), 'packing_update', context=context)
+                        list(purchase_ids), 'shipment_update', context=context)
         return res
 Move()
 
@@ -1808,9 +1923,9 @@ class Invoice(ModelSQL, ModelView):
         if isinstance(ids, (int, long)):
             ids = [ids]
         cursor.execute('SELECT id FROM purchase_invoices_rel ' \
-                'WHERE invoice IN (' + ','.join(['%s' for x in ids]) + ')',
+                'WHERE invoice IN (' + ','.join(('%s',) * len(ids)) + ')',
                 ids)
-        if cursor.rowcount:
+        if cursor.fetchone():
             self.raise_user_error(cursor, 'delete_purchase_invoice',
                     context=context)
         return super(Invoice, self).delete(cursor, user, ids,
@@ -1866,19 +1981,27 @@ class OpenSupplier(Wizard):
 OpenSupplier()
 
 
-class HandlePackingExceptionAsk(ModelView):
+class HandleShipmentExceptionAsk(ModelView):
     'Shipment Exception Ask'
-    _name = 'purchase.handle.packing.exception.ask'
+    _name = 'purchase.handle.shipment.exception.ask'
     _description = __doc__
 
     recreate_moves = fields.Many2Many(
         'stock.move', None, None, 'Recreate Moves',
-        domain="[('id', 'in', domain_moves)]", depends=['domain_moves'],
+        domain=["('id', 'in', domain_moves)"], depends=['domain_moves'],
         help='The selected moves will be recreated. '\
             'The other ones will be ignored.')
     domain_moves = fields.Many2Many(
         'stock.move', None, None, 'Domain Moves')
 
+    def init(self, cursor, module_name):
+        # Migration from 1.2: packing renamed into shipment
+        cursor.execute("UPDATE ir_model "\
+                "SET model = REPLACE(model, 'packing', 'shipment') "\
+                "WHERE model like '%%packing%%' AND module = %s",
+                (module_name,))
+        super(HandleShipmentExceptionAsk, self).init(cursor, module_name)
+
     def default_recreate_moves(self, cursor, user, context=None):
         return self.default_domain_moves(cursor, user, context=context)
 
@@ -1903,17 +2026,17 @@ class HandlePackingExceptionAsk(ModelView):
 
         return domain_moves
 
-HandlePackingExceptionAsk()
+HandleShipmentExceptionAsk()
 
-class HandlePackingException(Wizard):
+class HandleShipmentException(Wizard):
     'Handle Shipment Exception'
-    _name = 'purchase.handle.packing.exception'
+    _name = 'purchase.handle.shipment.exception'
     states = {
         'init': {
             'actions': [],
             'result': {
                 'type': 'form',
-                'object': 'purchase.handle.packing.exception.ask',
+                'object': 'purchase.handle.shipment.exception.ask',
                 'state': [
                     ('end', 'Cancel', 'tryton-cancel'),
                     ('ok', 'Ok', 'tryton-ok', True),
@@ -1961,9 +2084,9 @@ class HandlePackingException(Wizard):
                 context=context)
 
         purchase_obj.workflow_trigger_validate(cursor, user, data['id'],
-                'packing_ok', context=context)
+                'shipment_ok', context=context)
 
-HandlePackingException()
+HandleShipmentException()
 
 
 class HandleInvoiceExceptionAsk(ModelView):
@@ -1973,7 +2096,7 @@ class HandleInvoiceExceptionAsk(ModelView):
 
     recreate_invoices = fields.Many2Many(
         'account.invoice', None, None, 'Recreate Invoices',
-        domain="[('id', 'in', domain_invoices)]", depends=['domain_invoices'],
+        domain=["('id', 'in', domain_invoices)"], depends=['domain_invoices'],
         help='The selected invoices will be recreated. '\
             'The other ones will be ignored.')
     domain_invoices = fields.Many2Many(
